/* libdf3 - voxput.c */

#include "df3_priv.h"

#include <arpa/inet.h>
#include <string.h>


static bool validVoxel(size_t const bpv, uint32_t const val) {
  if (4 == bpv)
    return true;
  return (2 == bpv ? UINT16_MAX : UINT8_MAX) >= val;
}


df3Error df3_put_voxel(df3Handle *df3, df3Coord const at, uint32_t const val) {
  if (!df3  ||  !df3ValidHandle(df3))
    return ecBadHandle;
  else if (df3->readonly)
    return (df3->ec = ecBadArgument);
  else if (!df3ValidAt(df3->axes, at))
    return (df3->ec = ecBadVoxAt);
  else if (!validVoxel(df3->bpv, val))
    return (df3->ec = ecBadVoxVal);

  endianData u = {{0}};
  switch (df3->bpv) {
    case 4: u.v32 = htonl(val);           break;
    case 2: u.v16 = htons((uint16_t)val); break;
    case 1: u.v8 = (uint8_t)val;          break;
  }

  off_t const x = df3CalcFilepointer(df3, at);
  if (df3->ismapped) {
    mmFile * const p = df3->dp;
    p->dp[(size_t)x] = u.v8;
  }
  else if (-1 == fseek(df3->dp, x, SEEK_SET)) {
    df3->syserr = errno;
    return (df3->ec = ecSysErr);
  }
  else if (df3->bpv != fwrite(&u.raw, sizeof(uint8_t), df3->bpv, df3->dp)) {
    df3->syserr = errno;
    return (df3->ec = ecSysErr);
  }

  return ecOk;
}

/* ----------------------------------------------------------------- *
 *  this file is part of 'libdf3', part of the 'df3 tools' project.  *
 *  the content above is covered by the GPLv3+, see file COPYING.    *
 *  copyright (c) 2017 jr <creature.eternal@gmail.com>.              *
 *  all rights reserved.                                             *
 * ----------------------------------------------------------------- */
